/*
 * This file is part of OpenModelica.
 *
 * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
 * c/o Linkpings universitet, Department of Computer and Information Science,
 * SE-58183 Linkping, Sweden.
 *
 * All rights reserved.
 *
 * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3 LICENSE OR 
 * THIS OSMC PUBLIC LICENSE (OSMC-PL) VERSION 1.2. 
 * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE
 * OF THE OSMC PUBLIC LICENSE OR THE GPL VERSION 3, ACCORDING TO RECIPIENTS CHOICE. 
 *
 * The OpenModelica software and the Open Source Modelica
 * Consortium (OSMC) Public License (OSMC-PL) are obtained
 * from OSMC, either from the above address,
 * from the URLs: http://www.ida.liu.se/projects/OpenModelica or  
 * http://www.openmodelica.org, and in the OpenModelica distribution. 
 * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
 *
 * This program is distributed WITHOUT ANY WARRANTY; without
 * even the implied warranty of  MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
 * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
 *
 * See the full OSMC Public License conditions for more details.
 *
 * Main author: Wladimir Schamai, EADS Innovation Works / Linkping University, 2009-now
 *
 * Contributors: 
 *   Uwe Pohlmann, University of Paderborn 2009-2010, contribution to the Modelica code generation for state machine behavior, contribution to Papyrus GUI adaptations
 *   Parham Vasaiely, EADS Innovation Works / Hamburg University of Applied Sciences 2009-2011, implementation of simulation plugins
 */
package org.openmodelica.modelicaml.view.componentstree.dialogs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.openmodelica.modelicaml.common.instantiation.TreeObject;
import org.openmodelica.modelicaml.common.instantiation.TreeParent;
import org.openmodelica.modelicaml.common.instantiation.TreeUtls;
import org.openmodelica.modelicaml.common.services.ModelicaMLServices;
import org.openmodelica.modelicaml.common.utls.ResourceManager;

public class SearchDialog extends Dialog {
	private Tree tree;
	private Button btnSearch;
	private Text txtKeyWords;
	
	private TreeViewer viewer;
	private TreeParent root;
	
	private HashSet<String> keywords = new HashSet<String>();
	private Button btnOr;
	private Button btnAnd;
	
	private boolean searchInName = true;
	private boolean searchInComments = true;
	private boolean searchInTypeName = true;
	private boolean searchInTypeComments = false;

	public SearchDialog(Shell parentShell, TreeViewer viewer, TreeParent root) {
		
		super(parentShell);
		setShellStyle(SWT.SHELL_TRIM);
		
		this.root = root;
		this.viewer = viewer;
	}

	
	private HashSet<TreeObject> getAllTreeItems(TreeParent treeParent){
		HashSet<TreeObject> allTreeItems = new HashSet<TreeObject>();
		allTreeItems.add(treeParent);
		
		TreeObject[] children = treeParent.getChildren();
		for (int i = 0; i < children.length; i++) {
			allTreeItems.add(children[i]);
			if (children[i] instanceof TreeParent) {
				allTreeItems.addAll(getAllTreeItems((TreeParent)children[i]));
			}
		}
		
		return allTreeItems;
	}
	
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setImage(ResourceManager.getPluginImage("org.openmodelica.modelicaml.view.valuebindings","icons/find.png"));
		newShell.setText("Search in " + root.getName());		
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);
		container.setLayout(new GridLayout(1, false));
		container.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Group grpKeywords = new Group(container, SWT.NONE);
		grpKeywords.setText("Keywords search in names and comments");
		grpKeywords.setLayout(new GridLayout(3, false));
		GridData gd_grpKeywords = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_grpKeywords.heightHint = 64;
		grpKeywords.setLayoutData(gd_grpKeywords);
		
		txtKeyWords = new Text(grpKeywords, SWT.BORDER);
		txtKeyWords.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (txtKeyWords.getText().trim().length() > 1) {
					btnSearch.setEnabled(true);
				}
				else {
					btnSearch.setEnabled(false);
				}
			}
		});
		txtKeyWords.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		txtKeyWords.setFocus();
		
		btnSearch = new Button(grpKeywords, SWT.NONE);
		btnSearch.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				String[] keyWordsTextSplitted = txtKeyWords.getText().trim().split(" ");
				
				keywords.clear();
				for (String string : keyWordsTextSplitted) {
					keywords.add(string);
				}

				if (keywords.size() > 0) {
					// update the tree
					tree.removeAll();
					
					List<TreeObject> foundItems = getSortedByName(getMatchedItems()); 
					if (foundItems != null) {
						for (TreeObject treeObject : foundItems) {
							
							TreeItem treeItem = new TreeItem(tree, SWT.NONE);
							treeItem.setData(treeObject);
//							treeItem.setText(treeObject.getName());
							treeItem.setText(treeObject.getDotPath());
							
							if (treeObject.isRoot()) {
								treeItem.setImage(ResourceManager.getPluginImage("org.openmodelica.modelicaml.view.componentstree", "icons/Class.gif"));							
							}
							else {
								treeItem.setImage(ResourceManager.getPluginImage("org.openmodelica.modelicaml.view.componentstree", "icons/Property.gif"));
							}
						}
					}
				}
			}
		});
		btnSearch.setText("Search");
		
		btnOr = new Button(grpKeywords, SWT.RADIO);
		btnOr.setSelection(true);
		btnOr.setText("OR");
		
		btnAnd = new Button(grpKeywords, SWT.RADIO);
		btnAnd.setText("AND");
		new Label(grpKeywords, SWT.NONE);
		
		Group grpFoundItems = new Group(container, SWT.NONE);
		grpFoundItems.setText("Found items");
		grpFoundItems.setLayout(new GridLayout(1, false));
		grpFoundItems.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		tree = new Tree(grpFoundItems, SWT.BORDER | SWT.V_SCROLL);
		tree.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				TreeItem[] selectedItems = tree.getSelection();
				for (TreeItem selectedItem : selectedItems) {
					HashSet<Object> objects = TreeUtls.findTreeItems( ((TreeObject) selectedItem.getData()).getDotPath(), root, new HashSet<Object>());
					List<Object> newSelectionItems = new ArrayList<Object>();
					newSelectionItems.addAll(objects);
					viewer.setSelection(new StructuredSelection(newSelectionItems), true);
				}
			}
		});
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		return area;
	}

	
	
	public static List<TreeObject> getSortedByName(HashSet<TreeObject> set){
		if (set == null) { return null; }

		List<TreeObject> listSorted = new ArrayList<TreeObject>(set);
		Comparator<TreeObject> c = new Comparator<TreeObject>() {
			@Override
			public int compare(TreeObject arg0, TreeObject arg1) {
				return arg0.getName().compareToIgnoreCase(arg1.getName());
			}
		};
		Collections.sort(listSorted, c);
		
		return listSorted;
	}
	
	
	private HashSet<TreeObject> getMatchedItems(){
		HashSet<TreeObject> foundItems = new HashSet<TreeObject>();
		
		for (TreeObject treeObject: getAllTreeItems(root)) {
			String string = "";
			// search in tree object name, uml element qualified name, type qualified name, all comments 
			
			if (isSearchInName()) {
				string += treeObject.getName();
			}
			
			if (treeObject.getUmlElement() != null ){
				
				if (isSearchInName()) {
					string += ModelicaMLServices.getQualifiedName(treeObject.getUmlElement());
				}
				if (isSearchInComments()) {
					string += treeObject.getUmlElement().getOwnedComments().toString();							
				}

				if (treeObject.getUmlElement() instanceof Property) {
					Type type = ((Property)treeObject.getUmlElement()).getType();
					if (isSearchInTypeName()) {
						string +=  ModelicaMLServices.getQualifiedName(type);
					}
					if (isSearchInTypeComments()) {
						string += type.getOwnedComments().toString();
					}
				}
			}
			
			// OR search 
			if (btnOr.getSelection()) {
				for (String keyword : keywords) {
					// collect matches
					if (string.toLowerCase().contains(keyword.toLowerCase())) {
						foundItems.add(treeObject);
					}
				}
			}
			// AND search
			else if (btnAnd.getSelection()) {
				boolean matchedAlways = true;
//				boolean matched = false;
				for (String keyword : keywords) {
					// collect matches
					if (string.toLowerCase().contains(keyword.toLowerCase())) {
//						matched = true;
					}
					else {
//						matched = false;
						matchedAlways = false;
					}
				}
				if (matchedAlways) {
					foundItems.add(treeObject);
				}
			} 	
		}
		
		return foundItems;
	}
	
	/**
	 * Create contents of the button bar.
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
//		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
//				true);
//		createButton(parent, IDialogConstants.CANCEL_ID,
//				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(494, 373);
	}


	public boolean isSearchInComments() {
		return searchInComments;
	}


	public void setSearchInComments(boolean searchInComments) {
		this.searchInComments = searchInComments;
	}


	public boolean isSearchInName() {
		return searchInName;
	}


	public void setSearchInName(boolean searchInName) {
		this.searchInName = searchInName;
	}


	public boolean isSearchInTypeName() {
		return searchInTypeName;
	}


	public void setSearchInTypeName(boolean searchInTypeName) {
		this.searchInTypeName = searchInTypeName;
	}


	public boolean isSearchInTypeComments() {
		return searchInTypeComments;
	}


	public void setSearchInTypeComments(boolean searchInTypeComments) {
		this.searchInTypeComments = searchInTypeComments;
	}
}

